
// **********************************************************************
// *                                                                    *
// * ibmlm78.h -- definitions for the ibmlm78 module                    *
// *                                                                    *
// * Portions written by the IBM Director Development Team              *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

/* Other Portions:
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 */
 
#include <linux/ioctl.h>

/* version dependencies have been confined to a separate file */

#include "sysdep.h" 


/*
 * Macros to help debugging
 */

#undef PDEBUG             /* undef it, just in case */
#ifdef IBMLM78_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_DEBUG "ibmlm78: " fmt, ## args)
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */

#define IBMLM78_MAJOR 0   /* dynamic major by default */

#define IBMLM78_DEVS 4    /* ibmlm780 through ibmlm783 */

/*
 * The bare device is a variable-length region of memory.
 * Use a linked list of indirect blocks.
 *
 * "IbmLm78_Dev->data" points to an array of pointers, each
 * pointer refers to a memory page.
 *
 * The array (quantum-set) is IBMLM78_QSET long.
 */
#define IBMLM78_QUANTUM  4000 /* use a quantum size */
#define IBMLM78_QSET     500


#define KERNEL_DATABUF_SIZE 256         //d36489


typedef struct IbmLm78_Dev {
   void **data;
   struct IbmLm78_Dev *next;  /* next listitem */
   int vmas;                 /* active mappings */
   int quantum;              /* the current allocation size */
   int qset;                 /* the current array size */
   size_t size;              /* 32-bit will suffice */
   struct semaphore sem;     /* Mutual exclusion */
   spinlock_t lock;
} IbmLm78_Dev;

extern IbmLm78_Dev *ibmlm78_devices;

extern struct file_operations ibmlm78_fops;

// IOCTL Request Buffer
typedef struct Req_Buf {
    unsigned char addr;
    unsigned char command;
    unsigned long length;         /* int ??? FxFx */
    void          *data;
} Req_Buf;

#ifdef __x86_64__
// IOCTL Request Buffer (EM64T 32bit compatibility)
typedef struct Req_Buf_32 {
    u8   addr;
    u8  command;
    u32 length;         /* int ??? FxFx */
    u32 data;
                                                                                                                         
} Req_Buf_32;
#endif


/*
 * The different configurable parameters
 */
extern int ibmlm78_major;     /* main.c */
extern int ibmlm78_devs;
extern int ibmlm78_order;
extern int ibmlm78_qset;

/*
 * Prototypes for shared functions
 */
int ibmlm78_trim(IbmLm78_Dev *dev);
// IbmLm78_Dev *ibmlm78_follow(IbmLm78_Dev *dev, int n);



#ifndef min
#  define min(a,b) ((a)<(b) ? (a) : (b))
#endif

/*
 * Ioctl definitions
 */

/* Use 'K' as magic number */
#define IBMLM78_IOC_MAGIC  'K'

//----------MER till EOF--------
#define LM78C_IOCRESET    _IO(IBMLM78_IOC_MAGIC, 0)

#define LM78_WRITE_REGISTER            _IOW(IBMLM78_IOC_MAGIC,  1, unsigned int)
#define LM78_READ_REGISTER             _IOR(IBMLM78_IOC_MAGIC,  2, unsigned int)
#define LM78_SWITCH_307_GPIO           _IOR(IBMLM78_IOC_MAGIC,  3, unsigned int)
#define LM78_SWITCH_PIIX_GPO           _IOW(IBMLM78_IOC_MAGIC,  4, unsigned int)
#define PC8736x_READ_FAN_REGISTER      _IOR(IBMLM78_IOC_MAGIC,  5, unsigned int)
#define PC8736x_WRITE_FAN_REGISTER     _IOW(IBMLM78_IOC_MAGIC,  6, unsigned int)
#define S47B27x_READ_FAN_REGISTER      _IOR(IBMLM78_IOC_MAGIC,  7, unsigned int)
#define S47B27x_WRITE_FAN_REGISTER     _IOW(IBMLM78_IOC_MAGIC,  8, unsigned int)

#define RESET_ICH_CI_STATUS            _IOW(IBMLM78_IOC_MAGIC,  9, unsigned int)
#define READ_ICH_CI_STATUS             _IOR(IBMLM78_IOC_MAGIC,  10, unsigned int)
//#define READ_PF_DATA                   _IOR(IBMLM78_IOC_MAGIC,  11, unsigned int)
#define READ_PF_DATA                   _IOR(IBMLM78_IOC_MAGIC,  11, TLM78ReadParams)
#define SBRIDGE_WRITE_ASR_REGISTER     _IOW(IBMLM78_IOC_MAGIC,  12, unsigned int)
#define SBRIDGE_READ_ASR_REGISTER      _IOR(IBMLM78_IOC_MAGIC,  13, unsigned int)
#define W83627HF_READ_FAN_REGISTER     _IOR(IBMLM78_IOC_MAGIC,  14, unsigned int)
#define W83627HF_WRITE_FAN_REGISTER    _IOW(IBMLM78_IOC_MAGIC,  15, unsigned int)
#define H8_READ_RESET_REGISTER         _IOW(IBMLM78_IOC_MAGIC,  16, unsigned int)
#define SBRIDGE_READ_FAN_REGISTER      _IOR(IBMLM78_IOC_MAGIC,  17, unsigned int)
#define SBRIDGE_WRITE_FAN_REGISTER     _IOW(IBMLM78_IOC_MAGIC,  18, unsigned int)
#define SIS950_READ_FAN_REGISTER       _IOR(IBMLM78_IOC_MAGIC,  19, unsigned int)
#define SIS950_WRITE_FAN_REGISTER      _IOW(IBMLM78_IOC_MAGIC,  20, unsigned int)
#define LPC47M233_READ_REGISTER    	   _IOR(IBMLM78_IOC_MAGIC,  21, TLM78ReadParams)  
#define LPC47M233_WRITE_REGISTER   	   _IOW(IBMLM78_IOC_MAGIC,  22, TLM78WriteParams)

#define IBMLM78_IOCTEST1 _IOW(IBMLM78_IOC_MAGIC, 23, struct Req_Buf)
#define IBMLM78_IOCTEST2 _IOW(IBMLM78_IOC_MAGIC, 24, struct Req_Buf)  


#ifdef __x86_64__
#define LM78C_IOCRESET_32    _IO(IBMLM78_IOC_MAGIC, 0)
                                                                                                                         
#define LM78_WRITE_REGISTER_32            _IOW(IBMLM78_IOC_MAGIC,  1, unsigned int)
#define LM78_READ_REGISTER_32             _IOR(IBMLM78_IOC_MAGIC,  2, unsigned int)
#define LM78_SWITCH_307_GPIO_32           _IOR(IBMLM78_IOC_MAGIC,  3, unsigned int)
#define LM78_SWITCH_PIIX_GPO_32           _IOW(IBMLM78_IOC_MAGIC,  4, unsigned int)
#define PC8736x_READ_FAN_REGISTER_32      _IOR(IBMLM78_IOC_MAGIC,  5, unsigned int)
#define PC8736x_WRITE_FAN_REGISTER_32     _IOW(IBMLM78_IOC_MAGIC,  6, unsigned int)
#define S47B27x_READ_FAN_REGISTER_32      _IOR(IBMLM78_IOC_MAGIC,  7, unsigned int)
#define S47B27x_WRITE_FAN_REGISTER_32     _IOW(IBMLM78_IOC_MAGIC,  8, unsigned int)
                                                                                                                         
#define RESET_ICH_CI_STATUS_32            _IOW(IBMLM78_IOC_MAGIC,  9, unsigned int)
#define READ_ICH_CI_STATUS_32             _IOR(IBMLM78_IOC_MAGIC,  10, unsigned int)
//#define READ_PF_DATA                   _IOR(IBMLM78_IOC_MAGIC,  11, unsigned int)
#define READ_PF_DATA_32                   _IOR(IBMLM78_IOC_MAGIC,  11, TLM78ReadParams_32)
#define SBRIDGE_WRITE_ASR_REGISTER_32     _IOW(IBMLM78_IOC_MAGIC,  12, unsigned int)
#define SBRIDGE_READ_ASR_REGISTER_32      _IOR(IBMLM78_IOC_MAGIC,  13, unsigned int)
#define W83627HF_READ_FAN_REGISTER_32     _IOR(IBMLM78_IOC_MAGIC,  14, unsigned int)
#define W83627HF_WRITE_FAN_REGISTER_32    _IOW(IBMLM78_IOC_MAGIC,  15, unsigned int)
#define H8_READ_RESET_REGISTER_32         _IOW(IBMLM78_IOC_MAGIC,  16, unsigned int)
#define SBRIDGE_READ_FAN_REGISTER_32      _IOR(IBMLM78_IOC_MAGIC,  17, unsigned int)
#define SBRIDGE_WRITE_FAN_REGISTER_32     _IOW(IBMLM78_IOC_MAGIC,  18, unsigned int)
#define SIS950_READ_FAN_REGISTER_32       _IOR(IBMLM78_IOC_MAGIC,  19, unsigned int)
#define SIS950_WRITE_FAN_REGISTER_32      _IOW(IBMLM78_IOC_MAGIC,  20, unsigned int)
#define LPC47M233_READ_REGISTER_32        _IOR(IBMLM78_IOC_MAGIC,  21, TLM78ReadParams_32)
#define LPC47M233_WRITE_REGISTER_32       _IOW(IBMLM78_IOC_MAGIC,  22, TLM78WriteParams_32)
                                                                                                                         
#define IBMLM78_IOCTEST1_32    _IOW(IBMLM78_IOC_MAGIC, 23, struct Req_Buf_32)
#define IBMLM78_IOCTEST2_32    _IOW(IBMLM78_IOC_MAGIC, 24, struct Req_Buf_32)
#endif


// .....when add, then bump the 2 lines below

#define IBMLM78_IOCDEBUGCONTROL _IO(IBMLM78_IOC_MAGIC, 25)  /* debugging control */

#define IBMLM78_IOC_MAXNR 25

#define UCHAR unsigned char

//******************************************************************
// Input params to the LM78_READ_REGISTER function
typedef struct
{
   UCHAR addr;
   int length;
   UCHAR data;
   int param;
}  TReadLM78RegParams;

// Input params to the LM78_WRITE_REGISTER function
typedef struct
{
   UCHAR addr;
   PVOID data;
   int length;
   int param;
}  TWriteLM78RegParams;

typedef TWriteLM78RegParams TLM78WriteParams;
typedef TReadLM78RegParams TLM78ReadParams;


#ifdef __x86_64__
//******************************************************************
// Input params to the LM78_READ_REGISTER function (32 bit version)
typedef struct
{
   u8  addr;
   u32 length;
   u8  data;
   u32 param;
}  TReadLM78RegParams_32;
                                                                                                                         
// Input params to the LM78_WRITE_REGISTER function  (32 bit version)
typedef struct
{
   u8  addr;
   u32 data;
   u32 length;
   u32 param;
}  TWriteLM78RegParams_32;
                                                                                                                         
typedef TWriteLM78RegParams_32 TLM78WriteParams_32;
typedef TReadLM78RegParams_32 TLM78ReadParams_32;
#endif


// Input params to the PC8736x_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadPC8736xRegParams;

// Input params to the PC8736x_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWritePC8736xRegParams;

// Input params to the S47B27x_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadS47B27xRegParams;

// Input params to the S47B27x_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWriteS47B27xRegParams;

// Input params to the S47B27x_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadSIS950RegParams;

// Input params to the S47B27x_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWriteSIS950RegParams;

// Input params to the S47B27x_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadSBRIDGERegParams;

// Input params to the S47B27x_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWriteSBRIDGERegParams;

// Input params to the S47B27x_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadH8RegParams;

// Input params to the S47B27x_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWriteH8RegParams;

//next section added on 05/31/01 feature [] to add support for the Winbond
//W83627HF LPC Hardware monitoring chip
// Input params to the W83627HF_READ_FAN_REGISTER function
typedef TReadLM78RegParams  TReadW83627HFRegParams;

// Input params to the W83627HF_WRITE_FAN_REGISTER function
typedef TWriteLM78RegParams  TWriteW83627HFRegParams;

typedef struct
{
   UCHAR addr;
   UCHAR mask;       // Contains a 1 in each bit position to be changed
   UCHAR data;       // Sets bits indicated by mask onto the GPIO pins
}  TSwitch307GPIO;

typedef struct
{
   UCHAR addr;
   UCHAR data;
}  TSwitchPIIXGPO;

