
// **********************************************************************
// *                                                                    *
// * This is a common code file for the LM78 I/O.                       *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// ********************************************************************** 
 
#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#define __NO_VERSION__
#include <linux/module.h> /* get MOD_DEC_USE_COUNT, not the version string */
#include <linux/version.h> /* need it for conditionals */

#include <linux/kernel.h> /* printk() */
// #include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/fcntl.h>
#include <linux/tty.h>    /* current->tty */

#include "lm78io.h"
#include "ibmlm78-x.h"
#include "pci.h"

ULONG LPC47M233_detected = FALSE;
ULONG PC8736x_detected = FALSE;
ULONG PC87366_detected = FALSE;
ULONG S47B27x_detected = FALSE;
ULONG C47M13x_detected = FALSE;
ULONG VSBRIDGE_detected = FALSE;
ULONG PIIX4_detected = FALSE;
ULONG SIS950_detected = FALSE;
ULONG W83627HF_detected = FALSE;
ULONG H8_detected = FALSE;
ULONG LM78_detected = FALSE;
//next line added on 01/23/02 to add support for chassis intrusion on
//the ICH2
//Feature 17988:
//ICH2_detected flag name changed to ICHx_detected on 3/29/02 to
//accommodate detection of both ICH2 and ICH4 chip using the same flag.
ULONG ICHx_detected = FALSE;

ULONG LPC47M233_Base = 0;
ULONG PC8736x_Base = 0;
ULONG S47B27x_Base = 0;
ULONG C47M13x_Base = 0;
ULONG PIIX4_Base = 0;
ULONG VSBRIDGE_Fan_Base = 0;
ULONG VSBRIDGE_Chas_Base = 0;
ULONG SIS950_Base = 0;
ULONG PC87366_Temp_Base = 0;
ULONG PC87366_Volt_Base = 0;

ULONG LPC47M233_index = 0;
ULONG LPC47M233_data = 0;
ULONG W83627HF_Base = 0;
ULONG LM78_Base = 0;

ULONG H8_Base = 0;

//next line added on 01/23/02 to add support for chassis intrusion on
//the ICH2
//Feature 17988:
//ICH2_TCO_Base flag name changed to ICHx_TCO_Base on 3/29/02 to
//accommodate both ICH2 and ICH4 chips.
ULONG ICHx_TCO_Base = 0;

int CALLING_CONVENTION Check_LM78_Chip(void);

//------------------------------------------------------------------------------
void CALLING_CONVENTION InitLPC47CHip(
ULONG FanBaseAddr,
ULONG fannum)
{

   //KdPrint( ("LM78NT.SYS: checking the base Fan base_address [%d] and fannum [%d]\n",FanBaseAddr,fannum) );
	//setup FAN tach input
   OUTP(FanBaseAddr+0x34-fannum,((INP(FanBaseAddr+0x34-fannum) & 0x78) | 0x05));
	//setup FAN register
   //OUTP(FanBaseAddr+0x56+fannum,1);

	//set the FAN divisor to 8
   OUTP(FanBaseAddr+S47B27x_CtrlRegOffset,(INP(FanBaseAddr+S47B27x_CtrlRegOffset) | (0x30<<(fannum*2))));

   OUTP(FanBaseAddr+S47B27x_PME_EN1,(INP(FanBaseAddr+S47B27x_PME_EN1) | (0x40<<fannum)));

   OUTP(FanBaseAddr+S47B27x_PME_STS1,(UCHAR)(0x40<<fannum));  // Initial clear of status register
} //InitLPC47CHip



//------------------------------------------------------------------------------
//#if defined(WIN32) && !defined(WIN95)
//int CALLING_CONVENTION Check_W83627HF(PCHAR index, PCHAR data)
//#else
int CALLING_CONVENTION Check_W83627HF(ULONG index, ULONG data)
//#endif
{
   int retVal = TRUE;
// UCHAR bankSelect, vendorID, chipID, readBuffer;        // LinPort: readBuffer Unused Var
   UCHAR bankSelect, vendorID, chipID;

   OUTP(index,0x4e);
   bankSelect = (UCHAR) INP(data);
   OUTP(index,0x4f);
   vendorID = (UCHAR) INP(data);
   if ((int) bankSelect & 0x80)
   {
      if (vendorID != WINBOND_VENDOR_HIGH_ID)
         retVal = FALSE;
   }
   else
   {
      if (vendorID != WINBOND_VENDOR_LOW_ID)
         retVal = FALSE;
   }


   if (retVal)
   {
      OUTP(index,0x58);
      chipID = (UCHAR) INP(data);
      if (chipID != W83627HF_CHIP_ID)
         retVal = FALSE;
   }
   return (retVal);

}



//------------------------------------------------------------------------------
int CALLING_CONVENTION Check_SuperIO(
#if defined(WIN32) && !defined(WIN95)
PCHAR index,
PCHAR data,
#else
ULONG index,
ULONG data,
#endif
UCHAR deviceReg,
UCHAR deviceID)
{
   int retVal;

   OUTP(index,deviceReg);
   if (INP(data) == deviceID)
	   retVal = TRUE;
   else
	   retVal = FALSE;

   return (retVal);

}




//------------------------------------------------------------------------------
int CALLING_CONVENTION Check_S4Chip(
#if defined(WIN32) && !defined(WIN95)
PCHAR index,
PCHAR data
#else
ULONG index,
ULONG data
#endif
)
{
   // Enter configuration state
   OUTP(index, 0x55);


   if (Check_SuperIO(index,data,S47B27x_DEVICE_ID_REG, S47B27x_DEVICE_ID))
      S47B27x_detected = TRUE;
   else if (Check_SuperIO(index,data,S47B27x_DEVICE_ID_REG,C47M10x_DEVICE_ID) ||
            Check_SuperIO(index,data,S47B27x_DEVICE_ID_REG,C47M14x_DEVICE_ID))
      C47M13x_detected = TRUE;

   //next section added back on 04/15/02 to retrieve the base
   //address for the LPC47M13 and LPC47B27X chips - it is needed to
   //retrieve the data
   if (S47B27x_detected || C47M13x_detected)
   {
      // Set up SuperIO for access to PME
      OUTP(index, 0x07);
      OUTP(data, 0x0A);
      // Find the Base Address of the PME
      OUTP(index, 0x60);
      S47B27x_Base = INP(data) << 8;
      OUTP(index, 0x61);
      S47B27x_Base = S47B27x_Base | INP(data);
      // Activate PME
      OUTP(index, 0x30);
      OUTP(data, 0x01);

      // Exit configuration state
   }
   OUTP(index, 0xAA);
   return (S47B27x_detected || C47M13x_detected);

}



//------------------------------------------------------------------------------
int CALLING_CONVENTION Check_47M233Chip(
#if defined(WIN32) && !defined(WIN95)
PCHAR index,
PCHAR data
#else
ULONG index,
ULONG data
#endif
)
{
   int retVal = FALSE;
   // Enter configuration state
   OUTP(index, 0x55);


   //KdPrint( ("LM78NT.SYS: in the Check_47M233Chip section for index [%x] and data [%x]\n",index,data) );
   if (Check_SuperIO(index,data,LPC47M233_CHIP_ID_REG, LPC47M233_CHIP_ID))
   {
      //KdPrint( ("LM78NT.SYS: The 47M233 Chip was detected\n") );
      // Set up SuperIO for access to PME
      OUTP(index, 0x07);
      OUTP(data, 0x0A);
      // Find the Base Address of the PME
      OUTP(index, 0x60);
      LPC47M233_Base = INP(data) << 8;
      OUTP(index, 0x61);
      LPC47M233_Base = LPC47M233_Base | INP(data);
      // Activate PME
      OUTP(index, 0x30);
      OUTP(data, 0x01);
      retVal = TRUE;
   }
   //else
   //   KdPrint( ("LM78NT.SYS: The 47M233 Chip was not detected\n") );

   // Exit configuration state
   OUTP(index, 0xAA);
   
   return (retVal);

}



//------------------------------------------------------------------------------
void CALLING_CONVENTION Check_SISChip(
#if defined(WIN32) && !defined(WIN95)
PCHAR index,
PCHAR data
#else
ULONG index,
ULONG data
#endif
)
{
   // set up to enter MB PNP mode
   OUTP(index,0x87);
   OUTP(index,0x01);
   OUTP(index,0x55);
   OUTP(index,0x55);

   if (Check_SuperIO(index,data,SIS950_CHIP_ID1_REG,SIS950_CHIP_ID1) &&
       Check_SuperIO(index,data,SIS950_CHIP_ID2_REG,SIS950_CHIP_ID2))
   {
      SIS950_detected = TRUE;
      //set the LPN to environmental monitors
      OUTP(index,0x07);
      OUTP(data,0x04);

      //Find the Base Address of the Environmental Controller
      OUTP(index,0x60);
      SIS950_Base = INP(data) << 8;
      OUTP(index,0x61);
      SIS950_Base = SIS950_Base | INP(data);

      //activate Environmental Controller
      OUTP(index,0x30);
      OUTP(data,0x01);

      //exit configuration state
      OUTP(index,0x02);
      OUTP(data,0x02);
   }
}



//------------------------------------------------------------------------------
int CALLING_CONVENTION Check_LM78_Chip()
{
   int retVal = FALSE;
   UCHAR vid;
   UCHAR config;

   OUTP(LM78_Base, LM78_CONFIG);
   config = INP(LM78_Base+1);

   //KdPrint( ("LM78NT.SYS: checking the base base_port [%d]\n",(int)LM78_Base) );
   // If an LM78 is present, the MSB of the config register will be 0.  Also,
   // if the chip is started correctly, bit 3 (INT_CLEAR) will be 0.
   if ((config & 1) && (!(config & 8)) && (!(config & 0x80)))
   {
      // It looks like the LM78 is present and started
      retVal = TRUE;
   }
   else
   {
      // Store VID and fan divisor information before it gets blown away
      // by the reset
      OUTP(LM78_Base, LM78_VID);
      vid = INP(LM78_Base+1);

      // Perform a software reset of the LM-78
      // (write 0x80 to the configuration register)
      OUTP(LM78_Base, LM78_CONFIG);
      OUTP(LM78_Base+1, 0x80);

      // Verify the power on value
      if (INP(LM78_Base+1) == 0x08)
	  {
         // Start the A-D Converters
         // (write 0x01 to the configuration register)
         // outp(LM78_ADDR, LM78_CONFIG);   // unnecessary -- addr is latched
         OUTP(LM78_Base+1, 0x01);

         // Write VID and fan divisor information back to the LM78
         OUTP(LM78_Base, LM78_VID);
         OUTP(LM78_Base+1, vid);

         retVal = TRUE;
	  }
   }
   return (retVal);
}





//------------------------------------------------------------------------------
void CALLING_CONVENTION LM78_Init(PLOCAL_DEVICE_INFO pLDI)
{
 
// ULONG bus=0, device=0, retVal, f0bus=0, f0device=0;
   ULONG  retVal;
// ULONG  f0bus=0, f0device=0;		   // LinPort: Unused Var
   int    bus=0, device=0;             // LinPort: int needed for PCIWalk on linux
	
	
   //next line added on 03/19/02 defect 17722 to add another device
   //for the ICH2 check
   //Feature 17988: ich2device name changed to ichxdevice to accommodate
   //both ICH2 & ICH4 chips.
	
// ULONG ichxdevice=0;
   int    ichxdevice=0;                // LinPort: int needed for PCIWalk on linux
	
   int j;
   UCHAR readBuffer;
// ULONG vsbConfig = 0;				   // LinPort: Unused Var

   //printk("Start LM78Init..\n");	
	
   //KdPrint( ("LM78NT.SYS: about to walk the PCIBUS for VIA chip\n") );
   if (PCIWalk(VIA, SBRIDGE,4, &bus, &device))
      VSBRIDGE_detected = TRUE;

   if (PCIWalk(INTEL, PIIX4,4, &bus, &device))
      PIIX4_detected = TRUE;

   //next section added on 01/23/02 feature 16975 to add support
   //for the chassis intrusion - The base address is located at
   //function 0 device 31 - the ACPI_BASE. Chassis Intrusion is status
   //located at TCO_BASE + 0x06h and the TCO_BASE is ACPI_BASE + 0x60h
   //***********************************************************
   //next 2 lines changed on 03/19/02 to use another device instead
   //of the one shared with other chipsets. It is resetting the device
   //of others
   //device = 31;
   //if (PCIWalk(INTEL, ICH2,0, &bus, &device))
	//Feature 17988: ich2device changed to ichxdevice in order to
	//accommodate both ICH2 and ICH4 chip.
   ichxdevice = 31;
   if (PCIWalk(INTEL, ICH2,0, &bus, &ichxdevice) || PCIWalk(INTEL, ICH5,0, &bus, &ichxdevice))
      ICHx_detected = TRUE;
   //***********************************************************

   /**********************************************************
	Feature 17988:
	next section added on 03/29/02 feature ????? to add support
   for the ICH4 chip.
	The TCO logic is accessed via registers mapped to the PCI
	configuration space (Device 31:Function 0).
	The TCO I/O registers reside in a 32-byte range, pointed to by a
	TCOBASE value.  TCOBASE = ACPI + 60h in the PCI configuration space.
	The mapping for the TCO2_STS-TCO Status Register, within the
	32-byte range, is TCOBASE + 06h.*/
   ichxdevice = 31;
   if (PCIWalk(INTEL, ICH4,0, &bus, &ichxdevice))
      ICHx_detected = TRUE;
   /***********************************************************/

   if (Check_W83627HF(W83627HF_ADDR,W83627HF_DATA))
      W83627HF_detected = TRUE;

   if (!Check_S4Chip(S47B27x_INDEX, S47B27x_DATA))
      Check_SISChip(SIS950_INDEX, SIS950_DATA);
   //if (Check_SuperIO(S47B27x_INDEX, S47B27x_DATA, S47B27x_DEVICE_ID_REG, S47B27x_DEVICE_ID))
   //{
   //   S47B27x_detected = TRUE;
   //}

   //if (Check_SuperIO(S47B27x_INDEX, S47B27x_DATA, S47B27x_DEVICE_ID_REG, C47M10x_DEVICE_ID) ||
   //    Check_SuperIO(S47B27x_INDEX, S47B27x_DATA, S47B27x_DEVICE_ID_REG, C47M14x_DEVICE_ID))
   //   C47M13x_detected = TRUE;

   if (Check_SuperIO(PC8736x_INDEX, PC8736x_DATA, PC8736x_DEVICE_ID_REG, PC87360_DEVICE_ID) ||
       Check_SuperIO(PC8736x_INDEX, PC8736x_DATA, PC8736x_DEVICE_ID_REG, PC87363_DEVICE_ID))
      PC8736x_detected = TRUE;
       
   if (Check_SuperIO(PC8736x_INDEX, PC8736x_DATA, PC8736x_DEVICE_ID_REG, PC87369_DEVICE_ID))
   //{
      //KdPrint( ("LM78NT.SYS: 87366 chip detected\n") );

      PC87366_detected = TRUE;
   //}
   //else
   //   KdPrint( ("LM78NT.SYS: 87366 chip not detected\n") );


   LPC47M233_detected = Check_47M233Chip(LPC47M233_ADDR,LPC47M233_DATA);

   //if (Check_SuperIO(SIS950_INDEX, SIS950_DATA, SIS950_CHIP_ID1_REG, SIS950_CHIP_ID1) &&
   //    Check_SuperIO(SIS950_INDEX, SIS950_DATA, SIS950_CHIP_ID2_REG, SIS950_CHIP_ID2))
   //   SIS950_detected = TRUE;


   //if (LM78_detected)
   LM78_Base = LM78_ADDR;

   LM78_detected = Check_LM78_Chip();


   if (VSBRIDGE_detected)
   {
      VSBRIDGE_Chas_Base = (0xfffe & (ULONG)PCI_GetConfigRegister(bus,device,4,(0x48>>2)));

      VSBRIDGE_Fan_Base = (0xfffe & (ULONG)PCI_GetConfigRegister(bus,device,4,(0x70>>2)));
      //set the chip to start monitoring the hardware
      retVal = (0xf & (ULONG)PCI_GetConfigRegister(bus,device,4,(0x74>>2)));

      //set the fan divisor
      OUTP(VSBRIDGE_Fan_Base+0x47,0xf0);

      //the next 2 lines added on 01/23/02 defect 16950 to fix shutdown problem with POS
      //machines... The default was SMIs enabled and they should be disabled
      OUTP(VSBRIDGE_Fan_Base+0x43,0xff);
      OUTP(VSBRIDGE_Fan_Base+0x44,0xff);
   }

   //next section added on 01/23/02 feature 16975 to get the TCO_BASE address
   //located at ACPI_BASE+0x60h so the chassis intrusion bit can be monitored
   //Feature 17988:
	//ICH2_detected flag changed to ICHx_Detected to accommodate both ICH2 & ICH4 chips
	if (ICHx_detected)
   {
      //next line changed on 03/19/02 to use another device instead
      //of the one shared with other chipsets. It is resetting the device
      //of others
      //device = 31;
      //Feature 17988:
		//ich2device = 31; changed on 3/29/02 to ichxdevice to accommodate both
		//ICH2 and ICH4 chips
		ichxdevice = 31;
		//Feature 17988:
		//ICH2_TCO_Base change to ICHx_TCO_Base on 3/29/02 to accommodate both
		//ICH2 and ICH4 chips.
      ICHx_TCO_Base = 0x60 + (0xfffe & (ULONG)PCI_GetConfigRegister(bus,ichxdevice,0,(0x40>>2)));
   }


   if (PIIX4_detected)
   {
	   PIIX4_Base = (ULONG) PCI_GetConfigRegister(bus, device, 3, 16);  // Read Power Mgmt Base Address
	   PIIX4_Base &= 0xffc0;
   }

   if (W83627HF_detected)
   {
      W83627HF_Base = W83627HF_ADDR;
      OUTP(W83627HF_Base,0x40);
      readBuffer =  (UCHAR) INP(W83627HF_Base+1);
      if (!(readBuffer & 0x01))
		{
         readBuffer = readBuffer & 0x01;
         OUTP(W83627HF_Base+1,(int)readBuffer);
		}
   }

   if (S47B27x_detected)
   {

      //KdPrint( ("LM78NT.SYS: S47B27 detected\n") );
      // Set up  GPIOs for mux control of fans

      //next section changed by xxx
      //**************************************************************
      OUTP(S47B27x_Base + 0x28, (INP(S47B27x_Base + 0x28) & 0x78));
      OUTP(S47B27x_Base + 0x29, (INP(S47B27x_Base + 0x29) & 0x78));
      OUTP(S47B27x_Base + 0x2A, (INP(S47B27x_Base + 0x2A) & 0x78));
      //initialize Fan1
      InitLPC47CHip(S47B27x_Base, 0);
   }

   if (C47M13x_detected)
   {
      //KdPrint( ("LM78NT.SYS: 47M13 detected\n") );
      //initialize fan1 and fan2
      for (j=0;j<2;j++)
         InitLPC47CHip(S47B27x_Base, j);
   }


   if (PC8736x_detected || PC87366_detected)
   {
      OUTP(PC8736x_INDEX, 0x55);
      OUTP(PC8736x_INDEX, 0x07);
      OUTP(PC8736x_DATA, 0x09);

      // Disable the fan monitors
      OUTP(PC8736x_INDEX, 0xf0);
      OUTP(PC8736x_DATA, (INP(PC8736x_DATA) & 0xdb));

      if (PC87366_detected)
      {
         OUTP(PC8736x_INDEX, 0xf1);
         OUTP(PC8736x_DATA, (INP(PC8736x_DATA) & 0xfe));
      }
      
      // Find the Base Address of the fan monitor
      OUTP(PC8736x_INDEX, 0x60);
      PC8736x_Base = INP(PC8736x_DATA) << 8;
      OUTP(PC8736x_INDEX, 0x61);
      PC8736x_Base = PC8736x_Base  | INP(PC8736x_DATA);

      // Set up IO pins
      OUTP(PC8736x_INDEX, 0x22);
      OUTP(PC8736x_DATA, (UCHAR)(INP(PC8736x_DATA) | 0x14));
      
      if (PC87366_detected)
      {
         OUTP(PC8736x_INDEX, 0x07);
         OUTP(PC8736x_DATA, 0x0d);

         OUTP(PC8736x_INDEX, 0x60);
         PC87366_Volt_Base = INP(PC8736x_DATA) << 8;
         OUTP(PC8736x_INDEX, 0x61);
         PC87366_Volt_Base = PC87366_Volt_Base  | INP(PC8736x_DATA);

         OUTP(PC8736x_INDEX, 0x07);
         OUTP(PC8736x_DATA, 0x0e);

         OUTP(PC8736x_INDEX, 0x60);
         PC87366_Temp_Base = INP(PC8736x_DATA) << 8;
         OUTP(PC8736x_INDEX, 0x61);
         PC87366_Temp_Base = PC87366_Temp_Base  | INP(PC8736x_DATA);
      }
      
      OUTP(PC8736x_INDEX, 0xAA);

      // Enable fan monitor
      OUTP(PC8736x_INDEX, 0xf0);
      OUTP(PC8736x_DATA, (INP(PC8736x_DATA) | (0x04     )));  // first fan
      OUTP(PC8736x_INDEX, 0xf0);
      OUTP(PC8736x_DATA, (INP(PC8736x_DATA) | (0x04 << 3)));  // second fan
      
      if (PC87366_detected)
      {
         OUTP(PC8736x_INDEX, 0xf1);
         OUTP(PC8736x_DATA, (INP(PC8736x_DATA) | (0x01     )));  // first fan
      }

      // Set threshold and clock speed
      /*OUTP(PC8736x_Base + PC8736x_FMTHR    , 0xf0);  // first fan
      OUTP(PC8736x_Base + PC8736x_FMCSR    , 0x66);
      OUTP(PC8736x_Base + PC8736x_FMTHR + 3, 0xf0);  // second fan
      OUTP(PC8736x_Base + PC8736x_FMCSR + 3, 0x66);
      
      if (PC87366_detected)
      {
         OUTP(PC8736x_Base + PC8736x_FMTHR + 6, 0xf0);  // second fan
         OUTP(PC8736x_Base + PC8736x_FMCSR + 6, 0x66);
      }

      // Clear overflow and overthreshold bits
      OUTP(PC8736x_Base + PC8736x_FMCSR    , (INP(PC8736x_Base + PC8736x_FMCSR    ) | 0x06));
      OUTP(PC8736x_Base + PC8736x_FMCSR + 3, (INP(PC8736x_Base + PC8736x_FMCSR + 3) | 0x06));
      
      if (PC87366_detected)
         OUTP(PC8736x_Base + PC8736x_FMCSR + 6, (INP(PC8736x_Base + PC8736x_FMCSR + 6) | 0x06));
      */
  }

   if (SIS950_detected)
   {
      // set up to enter MB PNP mode
      /*OUTP(SIS950_INDEX,0x87);
      OUTP(SIS950_INDEX,0x01);
      OUTP(SIS950_INDEX,0x55);
      OUTP(SIS950_INDEX,0x55);

      //set the LPN to environmental monitors
      OUTP(SIS950_INDEX,0x07);
      OUTP(SIS950_DATA,0x04);

      //Find the Base Address of the Environmental Controller
      OUTP(SIS950_INDEX,0x60);
      SIS950_Base = INP(SIS950_DATA) << 8;
      OUTP(SIS950_INDEX,0x61);
      SIS950_Base = SIS950_Base | INP(SIS950_DATA);

      //activate Environmental Controller
      OUTP(SIS950_INDEX,0x30);
      OUTP(SIS950_DATA,0x01);

      //exit configuration state
      OUTP(SIS950_INDEX,0x02);
      OUTP(SIS950_DATA,0x02);*/

      //set the fan divisor
      OUTP(SIS950_Base+0x05,0x0b);
      OUTP(SIS950_Base+0x06,0x1b);

      //enable the fans
      OUTP(SIS950_Base+0x05,0x13);
      OUTP(SIS950_Base+0x06,0x70);
   }	
	

}


//------------------------------------------------------------------------------
LM78STATUS CALLING_CONVENTION ReadLM78(
UCHAR addr,
UCHAR *retValue,
ULONG IOCTL,
int param)
{
   UCHAR value = 0;
   ULONG base = 0;
// int retbase = 0;									// LinPort: retbase Unused Var
   
   //KdPrint( ("LM78NT.SYS: ReadLM78 Called\n") );
   debug_out(DETAILS_LVL,"ReadLM78.  Enter...\n");
   switch (IOCTL)
   {
/* FxFx: add this back in when add NEW extra parm and determine what LPC47M233_READ_REGISTER is and where.	   
      case LPC47M233_READ_REGISTER:
         if (!LPC47M233_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

         if ((addr > 0x55) && (addr < 0x6a) || addr <= 0x0f)
            value = INP(LPC47M233_Base + addr);
         else
         {
            OUTP(LPC47M233_Base+0x60, addr);
            value = INP(LPC47M233_Base+0x61);
         }
         break;
*/	   
      case W83627HF_READ_FAN_REGISTER:
         //KdPrint( ("LM78NT.SYS: W83627HF Called\n") );
	     debug_out(DETAILS_LVL,"ReadLM78.- W83627HF Called.\n");
         if (!W83627HF_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

         OUTP(W83627HF_Base, addr);
         value = INP(W83627HF_Base+1);

         break;
      //next section added on 01/23/02 feature 16975 to add
      //support to read the chassis intrusion status bit
      //*************************************************
      case READ_ICH_CI_STATUS:
			//Feature 17988:
			//ICH2_detected changed to ICHx_detected on 3/29/02
			//to accommodate both ICH2 & ICH4 chips
         if (!ICHx_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

			//Feature 17988:
			//ICH2_TCO_Base changed to ICHx_TCO_Base on 3/29/02
			//to accommodate both ICH2 & ICH4 chips.
         value = INP(ICHx_TCO_Base + addr);
         break;
      //*************************************************
      case LM78_READ_REGISTER:
          //KdPrint( ("LM78NT.SYS: LM78 Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- LM78 Called.\n");
		  if (!LM78_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

          OUTP(LM78_Base, addr);
          value = INP(LM78_Base+1);

		  break;
      case PC8736x_READ_FAN_REGISTER:
         //KdPrint( ("LM78NT.SYS: PC8736x Called\n") );
         debug_out(DETAILS_LVL,"ReadLM78. - PC8736x Called.\n");
         if (!PC8736x_detected && !PC87366_detected)
         //{
         //   KdPrint( ("LM78NT.SYS: 87366 chip not detected\n") );
            return (ERR_INVALID_DEVICE_REQUEST);
         //}
         //if ((addr < 0x00) || (addr > 0x20))
         //{
         //   KdPrint( ("LM78NT.SYS: addr for 87366 [%x] not valid\n",addr) );
         //   return ERR_INVALID_PARAMETER;
        // }
         //next section added on 05/06/03 defect [] to add support for the
         //pc87366 environmental sensor chip - fan, temp and volt are at 
         //separate base addresses so need to send them back to user for them
         //to use
         if (PC87366_detected)
         {
            switch (param)
            {
               case 0:
                  base = PC8736x_Base;
                  break;
               case 1:
                  base = PC87366_Temp_Base;
                  break;
               case 2:
                  base = PC87366_Volt_Base;
                  break;
            }
         }
         else
            base = PC8736x_Base;
         
         if (base != 0)
            value = INP(base + addr);
         else
            return ERR_INVALID_PARAMETER;

         break;
      case S47B27x_READ_FAN_REGISTER:
         //KdPrint( ("LM78NT.SYS: S47B27 Called\n") );
         debug_out(DETAILS_LVL,"ReadLM78.- S47B27 Called.\n");
         if (!S47B27x_detected && !C47M13x_detected)
         {
            //KdPrint( ("LM78NT.SYS: S47B27 Not Detected\n") );
            debug_out(DETAILS_LVL,"ReadLM78.- S47B27 Not Detected.\n");
			 
            return (ERR_INVALID_DEVICE_REQUEST);
         }
/*          if ((addr != S47B27x_PME_EN1)       &&
              (addr != S47B27x_PME_STS1)      &&
              (addr != S47B27x_MuxDataOffset) &&
              (addr != S47B27x_SpRegOffset)   &&
              //next line added by xxx
              (addr != S47B27x_SpRegOffset+1)   &&
              (addr != S47B27x_PreloadOffset) &&
              //next line added by xxx
              (addr != S47B27x_PreloadOffset+1) &&
              (addr != S47B27x_CtrlRegOffset) &&
              (addr != 0x33) &&
              (addr != 0x34) &&
              (addr != 0x56) &&
              (addr != 0x57) &&
              //(addr != 0x99) &&
              (addr != S47B27x_CtrlRegOffset))
          {
             //KdPrint( ("LM78NT.SYS: S47B27x returning bad parameter checking the base Fan base_address [%d] and addr [%d]\n",S47B27x_Base,addr) );
             return ERR_INVALID_PARAMETER;
          }
          else
          {*/
             value = INP(S47B27x_Base + addr);
             //KdPrint( ("LM78NT.SYS: S47B27x reached Fan base_address [%d], addr [%d], retval [%x]\n",S47B27x_Base,addr,value) );
          //}
         break;
      case H8_READ_RESET_REGISTER:
          //KdPrint( ("LM78NT.SYS: H8 Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- H8 Called.\n");
		  if (!H8_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);
		  value = INP(addr);
		  break;
      case SBRIDGE_READ_FAN_REGISTER:
          //KdPrint( ("LM78NT.SYS: Sbridge Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- Sbridge Called.\n");
		  if (!VSBRIDGE_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

		  if (((addr < 0x13) || (addr > 0x4d)) && addr != 0x4e)
             return ERR_INVALID_PARAMETER;

          //KdPrint( ("LM78NT.SYS: Processing SBRIDGE read\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- Processing SBRIDGE read.\n");
		  if (addr == 0x4e)
        {
           value = INP(VSBRIDGE_Chas_Base + addr);
           //KdPrint( ("LM78NT.SYS: Processing for chasbase SBRIDGE address [%x], retval [%d]\n",VSBRIDGE_Chas_Base + addr,value) );
        }
        else
        {
           value = INP(VSBRIDGE_Fan_Base + addr);
           //KdPrint( ("LM78NT.SYS: Processing for fanbase SBRIDGE address [%x], retval [%d]\n",VSBRIDGE_Fan_Base + addr,value) );
        }
		  break;
      case SBRIDGE_READ_ASR_REGISTER:
         //KdPrint( ("LM78NT.SYS: SBRIDGE_ASR Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- SBRIDGE_ASR Called.\n");
		  if (!VSBRIDGE_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

		  if (addr != 0x4d)
            return ERR_INVALID_PARAMETER;

          value = INP(VSBRIDGE_Chas_Base + addr);
		  break;
      case SIS950_READ_FAN_REGISTER:
          //KdPrint( ("LM78NT.SYS: SIS950 Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- SIS950 Called.\n");
		  if (!SIS950_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

		  if ((addr < 0x0b) || (addr > 0x45))
             return ERR_INVALID_PARAMETER;

          OUTP(SIS950_Base + 0x05, addr);
          value = INP(SIS950_Base + 0x06);
		  break;
      case READ_PF_DATA:
          //KdPrint( ("LM78NT.SYS: READ_PF Called\n") );
          debug_out(DETAILS_LVL,"ReadLM78.- READ_PF Called.\n");
		  OUTP(0x70,addr);
		  value = INP(0x71);
		  break;
      default:
        //KdPrint( ("LM78NT.SYS: default Called\n") );
        debug_out(DETAILS_LVL,"ReadLM78.- default Called.\n");
        return ERR_INVALID_PARAMETER;
   }
   *retValue = value;
   debug_out(DETAILS_LVL,"ReadLM78.  Value Read = %xh.\n", value);
   
   return (ERR_NO_ERROR);
}



//------------------------------------------------------------------------------
LM78STATUS CALLING_CONVENTION WriteLM78(
UCHAR addr,
UCHAR value,
ULONG IOCTL,
int param)
{
   ULONG base = 0;

   switch (IOCTL)
   {
/* FxFx: add this back in when add NEW extra parm and determine what LPC47M233_READ_REGISTER is and where.	   
      case LPC47M233_WRITE_REGISTER:
         if (!LPC47M233_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

         if ((addr > 0x55) && (addr < 0x6a) || addr <= 0x0f)
            OUTP(LPC47M233_Base + addr, value);
         else
         {
            OUTP(LPC47M233_Base+0x60, addr);
            OUTP(LPC47M233_Base+0x61, value);
         }
         break;
*/	   
      case LM78_WRITE_REGISTER:
		  if (!LM78_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

          OUTP(LM78_Base, addr);
          OUTP(LM78_Base+1, value);

		  break;
      case PC8736x_WRITE_FAN_REGISTER:
         if (!PC8736x_detected && !PC87366_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

         if (PC87366_detected)
         {
            switch (param)
            {
               case 0:
                  base = PC8736x_Base;
                  break;
               case 1:
                  base = PC87366_Temp_Base;
                  break;
               case 2:
                  base = PC87366_Volt_Base;
                  break;
            }
         }
         else
            base = PC8736x_Base;

         if (base)
            OUTP(base + addr, value);
         else
            return ERR_INVALID_PARAMETER;
         break;
      case S47B27x_WRITE_FAN_REGISTER:
        //KdPrint( ("LM78NT.SYS: S47B27 write Called\n") );
		  if (!S47B27x_detected && !C47M13x_detected)
        {
           //KdPrint( ("LM78NT.SYS: S47B27 write Called but not detected\n") );
		     return (ERR_INVALID_DEVICE_REQUEST);
        }
          if ((addr != S47B27x_PME_EN1)       &&
              (addr != S47B27x_PME_STS1)      &&
              (addr != S47B27x_MuxDataOffset) &&
              (addr != S47B27x_SpRegOffset)   &&
              //next line added by xxx
              (addr != S47B27x_SpRegOffset+1)   &&
              (addr != S47B27x_PreloadOffset) &&
              //next line added by xxx
              (addr != S47B27x_PreloadOffset+1) &&
              (addr != S47B27x_CtrlRegOffset) &&
              (addr != 0x33) &&
              (addr != 0x34) &&
              (addr != 0x56) &&
              (addr != 0x57) &&
              //(addr != 0x99) &&
              (addr != S47B27x_CtrlRegOffset))
          {
             //KdPrint( ("LM78NT.SYS: S47B27x write returning bad parameter checking the base Fan base_address [%d] and addr [%d]\n",S47B27x_Base,addr) );
             return ERR_INVALID_PARAMETER;
          }
          else
          {
             //KdPrint( ("LM78NT.SYS: S47B27x good parameter checking the base Fan base_address [%d] and addr [%d]\n",S47B27x_Base,addr) );
             OUTP(S47B27x_Base + addr, value);
          }
		  break;
      case RESET_ICH_CI_STATUS:
			//Feature 17988:
			//ICH2_detected changed to ICHx_detected on 3/29/02
			//to accommodate both ICH2 and ICH4 chips.
         if (!ICHx_detected)
            return (ERR_INVALID_DEVICE_REQUEST);

         //Feature 17988:
			//ICH2_TCO_Base changed to ICHx_TCO_Base on 3/29/02
			//to accommodate ICH2 and ICH4 chips.
			OUTP(ICHx_TCO_Base + addr,value);
         break;
      //*************************************************
	  case SBRIDGE_WRITE_ASR_REGISTER:
		  if (!VSBRIDGE_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

          if (addr != 0x4d)
            return ERR_INVALID_PARAMETER;

		  OUTP(VSBRIDGE_Chas_Base + addr, value);

		  break;
      case SBRIDGE_WRITE_FAN_REGISTER:
		  if (!VSBRIDGE_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

		  if (((addr < 0x13) || (addr > 0x40)) && addr != 0x4e)
            return ERR_INVALID_PARAMETER;

        //KdPrint( ("LM78NT.SYS: Processing SBRIDGE write\n") );
		  if (addr == 0x4e)
             OUTP(VSBRIDGE_Chas_Base + addr, value);
          else
             OUTP(VSBRIDGE_Fan_Base + addr, value);

		  break;
      case SIS950_WRITE_FAN_REGISTER:
		  if (!SIS950_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);
		  if ((addr < 0x0b) || (addr > 0x45))
             return ERR_INVALID_PARAMETER;

          OUTP(SIS950_Base + 0x05,addr);
          OUTP(SIS950_Base + 0x06, value);
		  break;
	  case W83627HF_WRITE_FAN_REGISTER:
		  if (!W83627HF_detected)
		     return (ERR_INVALID_DEVICE_REQUEST);

          OUTP(W83627HF_Base, addr);
          OUTP(W83627HF_Base+1, value);

		  break;
   }
   return (ERR_NO_ERROR);
}



//------------------------------------------------------------------------------
LM78STATUS Switch307GPIO(
UCHAR addr,
UCHAR mask,
UCHAR data)
{
   UCHAR value;

   value = INP(addr);
   value &= ~mask;
   value |= (data & mask);
   OUTP(addr, value);

   return (ERR_NO_ERROR);
}



//------------------------------------------------------------------------------
LM78STATUS SwitchPIIXGPO(
IN UCHAR addr,
IN UCHAR data)
{
                                // NOTE:  Uses METHOD_BUFFERED ioctls.
   ULONG mask;
// UCHAR value;				    // LinPort: value Unused Var
   ULONG gpo;

	if (!PIIX4_detected)
      return (ERR_INVALID_DEVICE_REQUEST);

   gpo = 0;
   gpo =   ((unsigned long)INP(PIIX4_Base+0x34));
   gpo |= (((unsigned long)INP(PIIX4_Base+0x35)) << 8);
   gpo |= (((unsigned long)INP(PIIX4_Base+0x36)) << 16);
   gpo |= (((unsigned long)INP(PIIX4_Base+0x37)) << 24);

   mask = (unsigned long)1 << addr;
   if (data == 0)
   {
      gpo &= (~mask);
   }
   else
   {
      gpo |= mask;
   }

   OUTP(PIIX4_Base+0x34, (UCHAR)(gpo & 0xFF));
   OUTP(PIIX4_Base+0x35, (UCHAR)((gpo>>8) & 0xFF));
   OUTP(PIIX4_Base+0x36, (UCHAR)((gpo>>16)&0xFF));
   OUTP(PIIX4_Base+0x37, (UCHAR)((gpo>>24)&0xFF));

   gpo = 0;
   gpo =   ((unsigned long)INP(PIIX4_Base+0x34));
   gpo |= (((unsigned long)INP(PIIX4_Base+0x35)) << 8);
   gpo |= (((unsigned long)INP(PIIX4_Base+0x36)) << 16);
   gpo |= (((unsigned long)INP(PIIX4_Base+0x37)) << 24);

   return (ERR_NO_ERROR);
}



