
// **********************************************************************
// *                                                                    *
// * This is a common code header for the LM78 I/O.                     *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// ********************************************************************** 

#ifndef LM78IO_H
#define LM78IO_H

#include "lm78os-x.h"

#define CALLING_CONVENTION


/* Default base port, and # of ports */
/* GPIO Ports */
#define BASE_PORT      0x295
#define NUMBER_PORTS       2

/* Base port for PC8736x Super I/O */
#define PC8736x_BASE_PORT       0x2e
#define PC8736x_NUMBER_PORTS       2
#define PC8736x_DEVICE_ID_REG   0x20
#define PC87360_DEVICE_ID       0xe1
#define PC87363_DEVICE_ID       0xe8
#define PC87369_DEVICE_ID       0xe9
#define PC8736x_FMTHR           0x06
#define PC8736x_FMSPR           0x07
#define PC8736x_FMCSR           0x08

/* Base port for 47B27x Super I/O */
#define S47B27x_BASE_PORT     0x2e
#define S47B27x_NUMBER_PORTS     2
#define S47B27x_DEVICE_ID_REG 0x20
#define S47B27x_DEVICE_ID     0x51
#define C47M10x_DEVICE_ID      0x59
#define C47M14x_DEVICE_ID      0x5f
#define S47B27x_PME_EN1       0x0a
#define S47B27x_PME_STS1      0x04
#define S47B27x_MuxDataOffset 0x4b
#define S47B27x_SpRegOffset   0x59
#define S47B27x_PreloadOffset 0x5b
#define S47B27x_CtrlRegOffset 0x58

//next section added on 4/21/2003 to add support for
//theLPC47M233 Super I/O Chip
/* Base port for LPC47M233 Super I/O */
#define LPC47M233_BASE_PORT     0x2e
#define LPC47M233_NUMBER_PORTS     2
#define LPC47M233_CHIP_ID_REG   0x20
#define LPC47M233_CHIP_ID       0x6b

/* Base port for SIS950 Super I/O */
#define SIS950_BASE_PORT     0x2e
#define SIS950_NUMBER_PORTS     2
#define SIS950_CHIP_ID1_REG  0x20
#define SIS950_CHIP_ID2_REG  0x21
#define SIS950_CHIP_ID1      0x87
#define SIS950_CHIP_ID2      0x05

/* Base port for H8 clear bit */
#define H8_BASE_PORT     0x434

#define LM78_CONFIG 0x40
#define LM78_VID    0x47

/* vendor ID high byte and low byte for Winbond */
#define W83627HF_BASE_PORT   0x295
#define WINBOND_VENDOR_HIGH_ID  0x5c
#define WINBOND_VENDOR_LOW_ID   0xa3
#define W83627HF_CHIP_ID        0x21



// FxFx: Verify these
#define LM78_ADDR               (BASE_PORT)
#define LM78_DATA               (BASE_PORT + 1)
#define PC8736x_INDEX           (PC8736x_BASE_PORT)
#define PC8736x_DATA            (PC8736x_BASE_PORT + 1)
#define S47B27x_INDEX           (S47B27x_BASE_PORT)
#define S47B27x_DATA            (S47B27x_BASE_PORT + 1)
#define SIS950_INDEX            (SIS950_BASE_PORT)
#define SIS950_DATA             (SIS950_BASE_PORT + 1)
#define W83627HF_ADDR           (W83627HF_BASE_PORT)
#define W83627HF_DATA           (W83627HF_BASE_PORT + 1)
#define LPC47M233_ADDR          (LPC47M233_BASE_PORT)
#define LPC47M233_DATA          (LPC47M233_BASE_PORT + 1)



// driver local data structure specific to each device object
typedef struct _LOCAL_DEVICE_INFO {
    ULONG               DeviceType;     // Our private Device Type
//    BOOLEAN             PortWasMapped;  // If TRUE, we have to unmap on unload
    PVOID               GPIOPortBase;   // base port address
    ULONG               GPIOPortCount;  // Count of I/O addresses used.
//    PVOID               PortBase;       // base port address
    ULONG               PortCount;      // Count of I/O addresses used.
//    ULONG               PortMemoryType; // HalTranslateBusAddress MemoryType
/*	
#if !defined (WIN95) && !defined (LINUXDEF)
	PDEVICE_OBJECT      DeviceObject;   // The LM78 device object.
//    UCHAR               CurrentPointer[8]; // Current pointer for each addr
    KSPIN_LOCK          spin_lock;      // Synchronize access to device
#endif
*/	
//    short               currentTemp;    // Most recently read temperature
    BOOLEAN             lm78_detected;  // If TRUE, LM78 chip is present
	BOOLEAN				PC8736x_detected;  // If TRUE, PC8736x Super I/O is present
	BOOLEAN				S47B27x_detected;  // If TRUE, 47B27x Super I/O is present
	//next section added on 08/16/00
	BOOLEAN				VSBRIDGE_detected;  // If TRUE, VSBRIDGE Super I/O is present
	BOOLEAN				SIS950_detected;  // If TRUE, SIS950 Super I/O is present
	BOOLEAN				W83627HF_detected;  // If TRUE, W83627HF Super I/O is present
	BOOLEAN				LPC47M233_detected;  // If TRUE, W83627HF Super I/O is present
	BOOLEAN				H8_detected;  // If TRUE, H8 is present
	ULONG				ChasPortBase;
    PVOID               lm78_PortBase;
	BOOLEAN             lm78_PortWasMapped;
    PVOID               W83627HF_PortBase;
	BOOLEAN             W83627HF_PortWasMapped;
    PVOID               PC8736x_PortBase;
	BOOLEAN             PC8736x_PortWasMapped;
    PVOID               S47B27x_PortBase;
	BOOLEAN             S47B27x_PortWasMapped;
    PVOID               SIS950_PortBase;
	BOOLEAN             SIS950_PortWasMapped;
    PVOID               LPC47M233_PortBase;
	BOOLEAN             LPC47M233_PortWasMapped;
} LOCAL_DEVICE_INFO, *PLOCAL_DEVICE_INFO;


// ------------
// Status codes   
// ------------
typedef enum {
   ERR_NO_ERROR = 0,
   ERR_NOT_IMPLEMENTED,		    // 1
   ERR_INVALID_DEVICE_REQUEST,	// 2
   ERR_INVALID_PARAMETER,	    // 3
   ERR_INSUFFICIENT_BUFFER      // 4	
} LM78STATUS;
               


// ----------------------------
// External Function prototypes
// ----------------------------

//void LM78Init(void FAR *port_base_address);
void CALLING_CONVENTION LM78_Init(PLOCAL_DEVICE_INFO pLDI);
LM78STATUS CALLING_CONVENTION WriteLM78(UCHAR addr, UCHAR value, ULONG IOCTL, int param);
LM78STATUS CALLING_CONVENTION ReadLM78(UCHAR addr,UCHAR *value, ULONG IOCTL, int param);
LM78STATUS SwitchPIIXGPO(UCHAR addr,UCHAR data);
LM78STATUS Switch307GPIO(UCHAR addr,UCHAR mask,UCHAR data);


#endif  // LM78IO_H
