
// **********************************************************************
// *                                                                    *
// * lm78main.c -- main module for IBM Linux LM78 driver                *
// *                                                                    *
// * Portions written by the IBM Director Development Team              *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

/* Other Portions:
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 */ 
 

#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>

/* modversion stuff: no #ifdef needed if 2.0 support is not needed */
// #ifdef CONFIG_MODVERSIONS
// #include <linux/modversions.h>
// #endif

#include <linux/kernel.h> /* printk() */
//#include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>        /* O_ACCMODE */

#include <asm/system.h>   /* cli(), *_flags */

#ifdef __x86_64__
#include <asm/ioctl32.h>
#endif

#include "lm78io.h"         /* local definitions */
#include "ibmlm78-x.h"      /* local definitions */


#ifdef __x86_64__
/*
 * The IOCTL cmd received from 32 bit compiled applications
 */
                                                                                                                         
/*
extern int register_ioctl32_conversion( unsigned int cmd,
                int(*handler)(unsigned int, unsigned int, unsigned long,
                        struct file* ));
extern int unregister_ioctl32_conversion( unsigned int cmd );
*/
#endif
 

/* Kmem caches were not available in 2.0. Disallow compilation in that case */
#ifdef LINUX_20
#  error "Kmem_cache functions are not available in Linux-2.0"
#else
#  if defined(LINUX_22) && LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18)
#    error "This module needs at least 2.2.18 to run"
#  endif
/* leave the #ifdef open, so 2.0 won't get other errors */


/*
 * No static symbols here, because register_symtab is called
 */

int ibmlm78_major =   IBMLM78_MAJOR;
int ibmlm78_devs =    IBMLM78_DEVS;    /* number of bare ibmlm78 devices */
int ibmlm78_qset =    IBMLM78_QSET;
int ibmlm78_quantum = IBMLM78_QUANTUM;

MODULE_PARM(ibmlm78_major, "i");
MODULE_PARM(ibmlm78_devs, "i");
MODULE_PARM(ibmlm78_qset, "i");
MODULE_PARM(ibmlm78_quantum, "i");
MODULE_AUTHOR("IBM");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)
	MODULE_LICENSE("GPL");
#endif
// The 2 key buffers that are used when talking with user space - MADE LOCAL TO IOCTL FUNCT
// Req_Buf reqbuf;
// unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];

IbmLm78_Dev *ibmlm78_devices; /* allocated in ibmlm78_init */

int ibmlm78_trim(IbmLm78_Dev *dev);

/* declare one cache pointer: use it for all devices */
kmem_cache_t *ibmlm78_cache;

unsigned int Debug_Level = 0;



#ifdef __x86_64__
//static int
int handler_ioctl_32(unsigned int fd, unsigned int cmd,
                        unsigned long arg, struct file *filp)
{

    debug_out(INTERFACE_LVL,"********* HIT 32 BIT HANDLER ***********\n");

    IbmLm78_Dev *dev = filp->private_data;
    int err= 0, ret = 0;
                                                                                                                         
    Req_Buf_32 reqbuf;
    unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];
                                                                                                                         
    unsigned char addr = 0;
    unsigned long len = 0;         /* int ??? FxFx  */
    void *data = 0;
    LM78STATUS status = 0;
                                                                                                                         
    //----------
    int retVal = 0;
    TReadLM78RegParams_32 lm78ReadParams_32;
    TWriteLM78RegParams_32 lm78WriteParams_32;
    TSwitch307GPIO tSwitchGPIOParams;
    TSwitchPIIXGPO tSwitchGPOParams;
    UCHAR buffer = 0;
    //----------
                                                                                                                         
    // Lock the driver via a Mutex to force serialize access by different processes
    if (down_interruptible (&dev->sem)) {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Signal received while blocking.\n");
                    return -ERESTARTSYS;
        }
    debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Enter.\n");
                                                                                                                         
        /* accept only root */
    if (! capable(CAP_SYS_ADMIN))  {ret = -EACCES; goto Exit_ioctl;}

    /* don't even decode wrong cmds: better returning  ENOTTY than EFAULT */
    if (_IOC_TYPE(cmd) != IBMLM78_IOC_MAGIC) {ret = -ENOTTY; goto Exit_ioctl;}
                                                                                                                         
    if (_IOC_NR(cmd) > IBMLM78_IOC_MAXNR) {ret = -ENOTTY; goto Exit_ioctl;}
                                                                                                                         
    /*
     * the type is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. Note that the type is user-oriented, while
     * verify_area is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
        err = !access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd));
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
        err =  !access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd));
    if (err) {ret = -EFAULT; goto Exit_ioctl;}
                                                                                                                         
    switch(cmd) {

      /*  USING THIS AS THE WAY TO SET DEBUG TRACE LEVEL */
      case IBMLM78_IOCTEST1_32:
        memset(&reqbuf, 0x00, sizeof(Req_Buf_32));
                memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);

       if (copy_from_user (&reqbuf, (Req_Buf_32 *)arg, sizeof(Req_Buf_32))) {ret = -EFAULT; goto Exit_ioctl;}
            // Reuse the addr field of the reqbuf as the way to specify the debug level
        //printk("Debug_Level before = %d\n", Debug_Level);
        Debug_Level = reqbuf.addr;
        //printk("Debug_Level after = %d\n", Debug_Level);
                                                                                                                         
        break;
                                                                                                                         
      /*  USING THIS AS THE KLOG TEST POINT (for user space synchronized logging) */
      case IBMLM78_IOCTEST2_32:
        memset(&reqbuf, 0x00, sizeof(Req_Buf_32));
                memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);

        // Get the user request
        if (copy_from_user (&reqbuf, (Req_Buf_32 *)arg, sizeof(Req_Buf_32))) {ret = -EFAULT; goto Exit_ioctl;}
        addr = reqbuf.addr;
            len = reqbuf.length;
                                                                                                                         
        // Get the user data to syslog (buffered I/O)
        if (copy_from_user (kernel_databuf, (unsigned char *)(unsigned long)reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
            data = &kernel_databuf[0];
                                                                                                                         
            // Write the data to the syslog if debug messages are enabled
        if (Debug_Level >= INTERFACE_LVL)
                printk("%s\n", (unsigned char *)data);
                                                                                                                         
        break;


      case W83627HF_READ_FAN_REGISTER_32:
          case LM78_READ_REGISTER_32:
      case PC8736x_READ_FAN_REGISTER_32:
          case S47B27x_READ_FAN_REGISTER_32:
      case H8_READ_RESET_REGISTER_32:
          case SBRIDGE_READ_FAN_REGISTER_32:
          case SBRIDGE_READ_ASR_REGISTER_32:
      case SIS950_READ_FAN_REGISTER_32:
      case READ_ICH_CI_STATUS_32:
      case READ_PF_DATA_32:
      case LPC47M233_READ_REGISTER_32:
             debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading register.ioctl.\n");
                                                                                                                         
         if (copy_from_user(&lm78ReadParams_32,(TReadLM78RegParams_32 *) arg, sizeof(TReadLM78RegParams_32)))
         {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading -> copy_from_user failed in ioctl\n");
            retVal = -EFAULT;
         }
         else
         {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading -> copy_from_user succeeded in ioctl\n");
//          spin_lock(&ibmlm78_devices->lock);
                                                                                                                         
            // if ((ReadLM78(lm78ReadParams.addr,&buffer,cmd)) == ERR_NO_ERROR)
            if ((ReadLM78(lm78ReadParams_32.addr, &buffer, cmd, lm78ReadParams_32.param)) == ERR_NO_ERROR)
            {
                   debug_out(INTERFACE_LVL,"ibmlm78_ioctl.- readlm78 succeeded\n");
                                                                                                                         
               lm78ReadParams_32.data = buffer;
               copy_to_user((TReadLM78RegParams_32 *)arg,&lm78ReadParams_32,sizeof(TReadLM78RegParams_32));
            }
            else
            {
                   debug_out(INTERFACE_LVL,"ibmlm78_ioctl.- readlm78 failed\n");
               retVal = -EIO;
            }
//          spin_unlock(&ibmlm78_devices->lock);
         }
         break;

                                                                                                                         
      case LM78_SWITCH_307_GPIO_32:
         if (copy_from_user(&tSwitchGPIOParams,(TSwitch307GPIO *) arg, sizeof(TSwitch307GPIO)))
            retVal = -EFAULT;
         else
         {
            spin_lock(&ibmlm78_devices->lock);
            Switch307GPIO(tSwitchGPIOParams.addr,tSwitchGPIOParams.mask,tSwitchGPIOParams.data);
            spin_unlock(&ibmlm78_devices->lock);
         }
         break;
                                                                                                                         
      case LM78_SWITCH_PIIX_GPO_32:
         if (copy_from_user(&tSwitchGPOParams,(TSwitchPIIXGPO *) arg, sizeof(TSwitchPIIXGPO)))
            retVal = -EFAULT;
         else
         {
            spin_lock(&ibmlm78_devices->lock);
            SwitchPIIXGPO(tSwitchGPOParams.addr,tSwitchGPOParams.data);
            spin_unlock(&ibmlm78_devices->lock);
         }
         break;
                                                                                                                         
      case LM78_WRITE_REGISTER_32:
      case PC8736x_WRITE_FAN_REGISTER_32:
      case S47B27x_WRITE_FAN_REGISTER_32:
      case SBRIDGE_WRITE_ASR_REGISTER_32:
      case SBRIDGE_WRITE_FAN_REGISTER_32:
      case SIS950_WRITE_FAN_REGISTER_32:
      case W83627HF_WRITE_FAN_REGISTER_32:
      case RESET_ICH_CI_STATUS_32:
      case LPC47M233_WRITE_REGISTER_32:
             debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  writing register.ioctl.\n");
                                                                                                                         
         if (copy_from_user(&lm78WriteParams_32,(TWriteLM78RegParams_32 *) arg,sizeof (TWriteLM78RegParams_32)))
            retVal = -EFAULT;
         else
         {
//          spin_lock(&ibmlm78_devices->lock);
            buffer = lm78ReadParams_32.data;
            // if ((WriteLM78(lm78ReadParams.addr,buffer,cmd)) != ERR_NO_ERROR)
            if ((WriteLM78(lm78WriteParams_32.addr, buffer, cmd, lm78WriteParams_32.param)) != ERR_NO_ERROR)
               retVal = -EIO;
//          spin_unlock(&ibmlm78_devices->lock);
         }
         break;



      default:  /* redundant, as cmd was checked against MAXNR */
        debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Switch default.\n");
        ret = -ENOTTY;
            goto Exit_ioctl;
    }
                                                                                                                         
    ret = status;
                                                                                                                         
Exit_ioctl:
    debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Exit RC=%x.\n\n", ret);
                                                                                                                         
        // Release the driver lock
    up (&dev->sem);
    return ret;

}
#endif



//---------------------------------------------------------------------------------------
void debug_out (unsigned int MsgLevel,  char *fmt, ...) 
{
    va_list  argptr;         /* pointer to additional args */ 
	char     buffer[256];    /* IMPORTANT: Keep message size within reason so don't overflow this local buffer */
	
    va_start(argptr, fmt);
    // printk("In debug_out!!!\n");
	
	if (Debug_Level >= MsgLevel) {
		printk("IBMLM78-PID=%x...", current->pid);
        vsprintf(buffer, fmt, argptr);
		printk(buffer);
	}	
    va_end(argptr);
}



/*
 * Open and close
 */
//---------------------------------------------------------------------------------------
int ibmlm78_open (struct inode *inode, struct file *filp)
{
    int num = MINOR(inode->i_rdev);
    IbmLm78_Dev *dev; /* device information */

    debug_out(INTERFACE_LVL,"ibmlm78_open.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))
	{
	    debug_out(INTERFACE_LVL,"ibmlm78_open.  Non-Root access attempt.\n");
		return -EACCES; 
	}

    /*  check the device number */
    if (num >= ibmlm78_devs) return -ENODEV;
    dev = &ibmlm78_devices[num];

    /* now trim to 0 the length of the device if open was write-only */
     if ( (filp->f_flags & O_ACCMODE) == O_WRONLY) {
        if (down_interruptible (&dev->sem))
            return -ERESTARTSYS;
        ibmlm78_trim(dev); /* ignore errors */
        up (&dev->sem);
    }

    /* and use filp->private_data to point to the device data */
    filp->private_data = dev;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    MOD_INC_USE_COUNT;
#endif
    return 0;          /* success */
}


//---------------------------------------------------------------------------------------
int ibmlm78_release (struct inode *inode, struct file *filp)
{
    debug_out(INTERFACE_LVL,"ibmlm78_release.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))
	{
	    debug_out(INTERFACE_LVL,"ibmlm78_release.  Non-Root access attempt.\n");
		return -EACCES; 
	}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
    MOD_DEC_USE_COUNT;
#endif
    return 0;
}



/*
 * The ioctl() implementation
 */
//---------------------------------------------------------------------------------------
int ibmlm78_ioctl (struct inode *inode, struct file *filp,
                 unsigned int cmd, unsigned long arg)
{
    IbmLm78_Dev *dev = filp->private_data;
    int err= 0, ret = 0;

    Req_Buf reqbuf;
    unsigned char kernel_databuf[KERNEL_DATABUF_SIZE];
	
    unsigned char addr = 0;
    unsigned long len = 0;         /* int ??? FxFx  */
    void *data = 0;
    LM78STATUS status = 0;	     
	
	//----------
    int retVal = 0;	
    TReadLM78RegParams lm78ReadParams;
    TWriteLM78RegParams lm78WriteParams;
    TSwitch307GPIO tSwitchGPIOParams;
    TSwitchPIIXGPO tSwitchGPOParams;
    UCHAR buffer = 0;	
	//---------- 
	
    // Lock the driver via a Mutex to force serialize access by different processes
    if (down_interruptible (&dev->sem)) {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Signal received while blocking.\n");
		    return -ERESTARTSYS;	
	}
    debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Enter.\n");
	
	/* accept only root */
    if (! capable(CAP_SYS_ADMIN))  {ret = -EACCES; goto Exit_ioctl;}	

    /* don't even decode wrong cmds: better returning  ENOTTY than EFAULT */
    if (_IOC_TYPE(cmd) != IBMLM78_IOC_MAGIC) {ret = -ENOTTY; goto Exit_ioctl;}
	    
    if (_IOC_NR(cmd) > IBMLM78_IOC_MAXNR) {ret = -ENOTTY; goto Exit_ioctl;}

    /*
     * the type is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. Note that the type is user-oriented, while
     * verify_area is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
        err = !access_ok(VERIFY_WRITE, (void *)arg, _IOC_SIZE(cmd));
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
        err =  !access_ok(VERIFY_READ, (void *)arg, _IOC_SIZE(cmd));
    if (err) {ret = -EFAULT; goto Exit_ioctl;}
	    
    // FxFx: Move these down inside the 2 cases IBMLM78_IOCTEST1 and IBMLM78_IOCTEST2
//  memset(&reqbuf, 0x00, sizeof(Req_Buf));
//  memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);
    
    switch(cmd) {
		
      /*  USING THIS AS THE WAY TO SET DEBUG TRACE LEVEL */	  
      case IBMLM78_IOCTEST1:
        memset(&reqbuf, 0x00, sizeof(Req_Buf));
		memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);		  
	  
       if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
	    // Reuse the addr field of the reqbuf as the way to specify the debug level
        //printk("Debug_Level before = %d\n", Debug_Level);
     	Debug_Level = reqbuf.addr; 
        //printk("Debug_Level after = %d\n", Debug_Level);
   
        break;
      
	  
      /*  USING THIS AS THE KLOG TEST POINT (for user space synchronized logging) */	  
      case IBMLM78_IOCTEST2:
        memset(&reqbuf, 0x00, sizeof(Req_Buf));
		memset(&kernel_databuf[0], 0x00, KERNEL_DATABUF_SIZE);			  
        // Get the user request 
        if (copy_from_user (&reqbuf, (Req_Buf *)arg, sizeof(Req_Buf))) {ret = -EFAULT; goto Exit_ioctl;}
     	addr = reqbuf.addr; 
	    len = reqbuf.length;
	
        // Get the user data to syslog (buffered I/O)
        if (copy_from_user (kernel_databuf, reqbuf.data, len)) {ret = -EFAULT; goto Exit_ioctl;}
	    data = &kernel_databuf[0]; 

 	    // Write the data to the syslog if debug messages are enabled
    	if (Debug_Level >= INTERFACE_LVL) 
	        printk("%s\n", (unsigned char *)data);

        break;

	
		
      case W83627HF_READ_FAN_REGISTER:
	  case LM78_READ_REGISTER:
      case PC8736x_READ_FAN_REGISTER:
	  case S47B27x_READ_FAN_REGISTER:
      case H8_READ_RESET_REGISTER:
	  case SBRIDGE_READ_FAN_REGISTER:
	  case SBRIDGE_READ_ASR_REGISTER:
      case SIS950_READ_FAN_REGISTER:
      case READ_ICH_CI_STATUS:
      case READ_PF_DATA:
      case LPC47M233_READ_REGISTER:		  
	     debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading register.ioctl.\n");
	  
         if (copy_from_user(&lm78ReadParams,(TReadLM78RegParams *) arg, sizeof(TReadLM78RegParams)))
         {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading -> copy_from_user failed in ioctl\n");
            retVal = -EFAULT;
         }
         else
         {
            debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  reading -> copy_from_user succeeded in ioctl\n");
//          spin_lock(&ibmlm78_devices->lock);

            // if ((ReadLM78(lm78ReadParams.addr,&buffer,cmd)) == ERR_NO_ERROR)
            if ((ReadLM78(lm78ReadParams.addr, &buffer, cmd, lm78ReadParams.param)) == ERR_NO_ERROR)
            {
	           debug_out(INTERFACE_LVL,"ibmlm78_ioctl.- readlm78 succeeded\n");
				
               lm78ReadParams.data = buffer;
               copy_to_user((TReadLM78RegParams *)arg,&lm78ReadParams,sizeof(TReadLM78RegParams));
            }
            else
            {
	           debug_out(INTERFACE_LVL,"ibmlm78_ioctl.- readlm78 failed\n");
               retVal = -EIO;
            }
//          spin_unlock(&ibmlm78_devices->lock);
         }
         break;
		 
		 
		 
		 
      case LM78_SWITCH_307_GPIO:
         if (copy_from_user(&tSwitchGPIOParams,(TSwitch307GPIO *) arg, sizeof(TSwitch307GPIO)))
            retVal = -EFAULT;
         else
         {
            spin_lock(&ibmlm78_devices->lock);
            Switch307GPIO(tSwitchGPIOParams.addr,tSwitchGPIOParams.mask,tSwitchGPIOParams.data);
            spin_unlock(&ibmlm78_devices->lock);
         }
         break;
		 
		 
		 
      case LM78_SWITCH_PIIX_GPO:
         if (copy_from_user(&tSwitchGPOParams,(TSwitchPIIXGPO *) arg, sizeof(TSwitchPIIXGPO)))
            retVal = -EFAULT;
         else
         {
            spin_lock(&ibmlm78_devices->lock);
            SwitchPIIXGPO(tSwitchGPOParams.addr,tSwitchGPOParams.data);
            spin_unlock(&ibmlm78_devices->lock);
         }
         break;
		 
		 

      case LM78_WRITE_REGISTER:
      case PC8736x_WRITE_FAN_REGISTER:
      case S47B27x_WRITE_FAN_REGISTER:
      case SBRIDGE_WRITE_ASR_REGISTER:
      case SBRIDGE_WRITE_FAN_REGISTER:
      case SIS950_WRITE_FAN_REGISTER:
      case W83627HF_WRITE_FAN_REGISTER:
      case RESET_ICH_CI_STATUS:
      case LPC47M233_WRITE_REGISTER:	   	
	     debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  writing register.ioctl.\n");
	  
         if (copy_from_user(&lm78WriteParams,(TWriteLM78RegParams *) arg,sizeof (TWriteLM78RegParams)))
            retVal = -EFAULT;
         else
         {
//          spin_lock(&ibmlm78_devices->lock);
            buffer = lm78ReadParams.data;
            // if ((WriteLM78(lm78ReadParams.addr,buffer,cmd)) != ERR_NO_ERROR)
            if ((WriteLM78(lm78WriteParams.addr, buffer, cmd, lm78WriteParams.param)) != ERR_NO_ERROR)
               retVal = -EIO;
//          spin_unlock(&ibmlm78_devices->lock);
         }
         break;		

		 
       
      default:  /* redundant, as cmd was checked against MAXNR */
        debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Switch default.\n");
        ret = -ENOTTY; 
	    goto Exit_ioctl;	   
    }

    ret = status;
	
Exit_ioctl:	
    debug_out(INTERFACE_LVL,"ibmlm78_ioctl.  Exit RC=%x.\n\n", ret);

	// Release the driver lock
    up (&dev->sem);
    return ret;

}


// File operations
//---------------------------------------------------------------------------------------
struct file_operations ibmlm78_fops = {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
    owner : THIS_MODULE,
#endif
    ioctl: ibmlm78_ioctl,
    open:  ibmlm78_open,
    release: ibmlm78_release,
};



// Utility functions 
//---------------------------------------------------------------------------------------
int ibmlm78_trim(IbmLm78_Dev *dev)
{
    IbmLm78_Dev *next, *dptr;
    int qset = dev->qset;   /* "dev" is not-null */
    int i;

    debug_out(INTERFACE_LVL,"ibmlm78_trim.  Enter.\n");
	
    if (dev->vmas) /* don't trim: there are active mappings */
        return -EBUSY;

    for (dptr = dev; dptr; dptr = next) { /* all the list items */
        if (dptr->data) {
            for (i = 0; i < qset; i++)
                if (dptr->data[i])
                    kmem_cache_free(ibmlm78_cache, dptr->data[i]);

            kfree(dptr->data);
            dptr->data=NULL;
        }
        next=dptr->next;
        if (dptr != dev) kfree(dptr); /* all of them but the first */
    }
    dev->size = 0;
    dev->qset = ibmlm78_qset;
    dev->quantum = ibmlm78_quantum;
    dev->next = NULL;
    return 0;
}




/*
 * Module load and Unload
 */
//---------------------------------------------------------------------------------------
int ibmlm78_init(void)
{
    int result, i;

    //printk("IBMLM78:ibmlm78_init.  Enter.\n");

    SET_MODULE_OWNER(&ibmlm78_fops);
    /*
     * Register your major, and accept a dynamic number
     */
    result = register_chrdev(ibmlm78_major, "ibmlm78", &ibmlm78_fops);
    if (result < 0) return result;
    if (ibmlm78_major == 0) ibmlm78_major = result; /* dynamic */
		
    //printk("IBMLM78:ibmlm78_init.  ibmlm78_major = %d\n", ibmlm78_major);
	
    /* 
     * allocate the devices -- we can't have them static, as the number
     * can be specified at load time
     */
    ibmlm78_devices = kmalloc(ibmlm78_devs * sizeof (IbmLm78_Dev), GFP_KERNEL);
    if (!ibmlm78_devices) {
        result = -ENOMEM;
        goto fail_malloc;
    }
    memset(ibmlm78_devices, 0, ibmlm78_devs * sizeof (IbmLm78_Dev));
    for (i=0; i < ibmlm78_devs; i++) {
        ibmlm78_devices[i].quantum = ibmlm78_quantum;
        ibmlm78_devices[i].qset = ibmlm78_qset;
        sema_init (&ibmlm78_devices[i].sem, 1);
    }

    /* init_module: create a cache for our quanta */
    ibmlm78_cache =
	kmem_cache_create("ibmlm78", ibmlm78_quantum,
			  0, SLAB_HWCACHE_ALIGN,
			  NULL, NULL); /* no ctor/dtor */
    if (!ibmlm78_cache) {
        result = -ENOMEM;
        goto fail_malloc2;
    }

	// Initialize the hardware
    LM78_Init(0);                 					
	
#ifdef __x86_64__
        /*
         * Register the 32-bit ioctl conversion if on 64bit systems
         */
        register_ioctl32_conversion( IBMLM78_IOCTEST1_32, handler_ioctl_32 );
        register_ioctl32_conversion( READ_PF_DATA_32, handler_ioctl_32 );
        register_ioctl32_conversion( LPC47M233_READ_REGISTER_32, handler_ioctl_32 );
        register_ioctl32_conversion( LPC47M233_WRITE_REGISTER_32, handler_ioctl_32 );
#endif

	
    //printk("IBMLM78:ibmlm78_init.  Exit - Good.\n");
    return 0; /* succeed */

  fail_malloc2:
    kfree(ibmlm78_devices);
  fail_malloc:
    unregister_chrdev(ibmlm78_major, "ibmlm78");

    //printk("IBMLM78:ibmlm78_init.  Exit - Fail.\n");
    return result;
}


//---------------------------------------------------------------------------------------
void ibmlm78_cleanup(void)
{
    int i;

    debug_out(INTERFACE_LVL, "ibmlm78_cleanup.  Enter.\n");

#ifdef __x86_64__
    /*
     * Un-Register the 32-bit ioctl conversion if on 64bit systems
     */
    unregister_ioctl32_conversion( IBMLM78_IOCTEST1_32 );
    unregister_ioctl32_conversion( READ_PF_DATA_32 );
    unregister_ioctl32_conversion( LPC47M233_READ_REGISTER_32 );
    unregister_ioctl32_conversion( LPC47M233_WRITE_REGISTER_32 );
#endif

    unregister_chrdev(ibmlm78_major, "ibmlm78");

    for (i=0; i<ibmlm78_devs; i++)
        ibmlm78_trim(ibmlm78_devices+i);
    kfree(ibmlm78_devices);

    /* cleanup_module: release the cache of our quanta */
    kmem_cache_destroy(ibmlm78_cache);

    debug_out(INTERFACE_LVL,"ibmlm78_cleanup.  Exit.\n");
}

#endif /* not linux-2.0 */


//---------------------------------------------------------------------------------------
module_init(ibmlm78_init);
module_exit(ibmlm78_cleanup);
