// **********************************************************************
// *                                                                    *
// * Retrieving information about PCI devices using the configuration   *
// * registers.                                                         *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************
 
// LinPort: added these
#include <asm/io.h>
#define INPD(x) inl(x)
#define OUTPD(x,y) outl(y,x)


//#include <stdio.h>
//#include "defines.h"     
#include "pci.h"
char *ibm_copyright_string = "Portions Copyright IBM Corp, 1997-2003  All Rights Reserved.";


// Walk the PCI Bus looking for Vendor ID (VID) and Device Id (DID)
// at function 0 matching the parameters.  Returns the PCI bus and
// PCI device number of the device, if found.
// Return value:  1 if found, 0 not found
//next section changed on 08/10/00 feature 8529 to add the function to the parameter list
int PCIWalk(int IN vid, int IN did, int IN function, 
				int OUT FAR *bus, int OUT FAR *device)
//int PCIWalk(int IN vid, int IN did,
//            int OUT FAR *bus, int OUT FAR *device)
{
   int i, j;
   unsigned long data=0;

   unsigned long match = (((unsigned long)did & 0xffff) << 16) |
                         ((unsigned long)vid & 0xffff);

   //printk("PCIWALK match [%x]\n",(unsigned int)match);
   for (i=0; i<PCI_MAX_BUSES; i++)
   {
      for (j=0; j<PCI_MAX_DEVICES; j++)
      {
         //printk("PCIWALK match [%x]\n",(unsigned int)match);
         data = PCI_GetConfigRegister(i, j, function, 0);
         //printk("PCIWALK match [%x] and data [%x]\n",(int)match,(int)data);
         if (data == match)
         {
             //PDEBUG("PCIWALK match found\n");
            *bus = i;
            *device = j;
            return 1;
         }
         //else
            //PDEBUG("PCIWALK match not found\n");
           
      }
   }

   return 0;
}


// target is a DoubleWord offset into the data registers (not a BYTE offset!)
unsigned long PCI_GetConfigRegister(int IN bus, int IN device,
                                    int IN fctn, int IN target)
{
   unsigned long addr;
   unsigned long retVal = 0;

   PCI_ADDR_INIT(&addr);
   PCI_ADDR_SET_BUS(&addr, bus);
   PCI_ADDR_SET_DEVICE(&addr, device);
   PCI_ADDR_SET_FCTN(&addr, fctn);
   PCI_ADDR_SET_TARGET(&addr, target);
   //printk("PCI_GetConfigRegister addr [%x]\n",(unsigned int)addr);
   OUTPD(PCI_CONFIG_ADDR, addr);
   retVal = INPD(PCI_CONFIG_DATA);
   //printk("PCI_GetConfigRegister retval [%x]\n",(unsigned int)retVal);
   return (retVal);

}


// target is a DoubleWord offset into the data registers (not a BYTE offset!)
void PCI_SetConfigRegister(int IN bus, int IN device,
                           int IN fctn, int IN target,
                           unsigned long data)
{
   unsigned long addr;

   PCI_ADDR_INIT(&addr);
   PCI_ADDR_SET_BUS(&addr, bus);
   PCI_ADDR_SET_DEVICE(&addr, device);
   PCI_ADDR_SET_FCTN(&addr, fctn);
   PCI_ADDR_SET_TARGET(&addr, target);
   OUTPD(PCI_CONFIG_ADDR, addr);
   OUTPD(PCI_CONFIG_DATA, data);
   return;
}

/*
void main()
{
   int bus, device;
   int status;
   unsigned long smBaseAddr;

// Search for PIIX4 device
//   status = PCIWalk(0x8086, 0x7110, &bus, &device);
//   status = PCIWalk(0x8086, 0x1229, &bus, &device);

   if (status)
      printf("found on bus %d device %d\n", bus, device);
   else
      printf("device not found\n");

   if (status)
   {
      smBaseAddr = PCI_GetConfigRegister(bus, device, 3, 0x24);
   }
}
*/
