// **********************************************************************
// *                                                                    *
// * Retrieving information about PCI devices using the configuration   *
// * registers.                                                         *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************
 
#ifndef PCI_H
#define PCI_H

/*
#if defined(WIN95)
   #include <windows.h>
   #include <winbase.h>
   #include <conio.h>
#elif defined(WIN32)
   #include <ntddk.h>
   #define CALLING_CONVENTION  __cdecl
   #define EXPORT __declspec(dllexport)
#endif
*/

//typedef unsigned char BYTE;

// I/O ports for accessing PCI Configuration registers
#define PCI_CONFIG_ADDR 0x0CF8
#define PCI_CONFIG_DATA 0x0CFC  

// PCI Configuration values
#define INTEL 0x8086
#define PIIX4 0x7110

//next line added on 01/23/02 feature 16975 to add support for the ICH2 intruder
//bit
#define ICH2      0x2440

/*Feature 17988:
next line added on (3/29/02) to add support for the ICH4 chip.
DID (Device ID) for Low Pin Count (LPC) bridge (Function 0)*/
#define ICH4      0x24C0
#define ICH5      0x24D0  // DID for ICH3 Function 3

//next section added on 08/10/00 feature 8529 to add support for the exterminator
//*******************************************
#define VIA			0x1106
#define SBRIDGE	0x3057
//*******************************************

// Misc defines
#define PCI_MAX_BUSES 7
#define PCI_MAX_DEVICES 32

// Documentation macros
#define IN
#define OUT
#define FAR
/*
#if defined(WIN95)
#pragma message("win95")   
   #pragma intrinsic(_inp,_inpw,_inpd,_outp,_outpw,_outpd)
   #define INP(x)       ((BYTE)_inp((unsigned short)(x)))
   #define OUTP(x,y)    _outp((unsigned short)(x),(int)(y))
   #define INPD(x)      _inpd((unsigned short)(x))
   #define OUTPD(x,y)   _outpd((unsigned short)(x),(int)(y))

#elif defined(WIN32)
#pragma message("win32")   
   #define OUTPD(port, data)  _outpd((port), (data))
   #define INPD(port)         _inpd((port))
   #define INP(x)       (BYTE)_inp((unsigned short)(x))
   #define OUTP(x,y)    (BYTE)_outp((unsigned short)(x),(int)(y))
#endif
*/
// Macros
#define PCI_ADDR_INIT(addr)               (*(addr)) = 0x80000000
#define PCI_ADDR_SET_BUS(addr, bus)       (*(addr)) |= (((long)bus & 0xffL) << 16)
#define PCI_ADDR_SET_DEVICE(addr, dev)    (*(addr)) |= (((long)dev & 0x1fL) << 11)
#define PCI_ADDR_SET_FCTN(addr, fctn)     (*(addr)) |= (((long)fctn & 0x7L) << 8)
#define PCI_ADDR_SET_TARGET(addr, offset) (*(addr)) |= (((long)offset & 0x3fL) << 2)


int PCIWalk(int IN vid, int IN did, int IN function, int OUT *bus, int OUT *device);
unsigned long PCI_GetConfigRegister(int IN bus, int IN device,
                                    int IN fctn, int IN target);

void PCI_SetConfigRegister(int IN bus, int IN device,
                           int IN fctn, int IN target, unsigned long IN data);


#endif // PCI_H

